﻿using System;

namespace AssetGraph.Core.Attributes
{
    [AttributeUsage(AttributeTargets.Class)]
    public class NodeInfoAttribute : Attribute
    {
        private string name;
        private string menuItem;
        private string ussPath;
        private Type editorType;
        private string docuementationURL;
        
        public string Title => name;
        public string MenuItem => menuItem;
        public string UssPath => ussPath;
        public Type EditorType => editorType;
        public string DocumentationationURL => docuementationURL;

        public NodeInfoAttribute(string name, string menuItem = "", bool requiresImage = false, string ussPath = null, Type editorType = null, string docuementationURL = null)
        {
            this.name = name;
            this.menuItem = menuItem;
            this.ussPath = ussPath;
            this.editorType = editorType;
            this.docuementationURL = docuementationURL;
        }
    }
    
    // Base type for Input and Output attributes
    public abstract class PortAttribute : Attribute
    {
        public string Label { get; }

        protected PortAttribute(string label)
        {
            Label = label;
        }
    }

    // Input attribute inherits from base type
    [AttributeUsage(AttributeTargets.Field)]
    public class Input : PortAttribute
    {
        public Input(string label) : base(label) { }
    }

    // Output attribute inherits from base type
    [AttributeUsage(AttributeTargets.Field)]
    public class Output : PortAttribute
    {
        public Output(string label) : base(label) { }
    }
}