using System;
using UnityEditor;
using UnityEditor.Experimental.GraphView;
using UnityEngine;

namespace AssetGraph.Core
{
    public class AssetGraphEditorWindow : EditorWindow
    {
        [SerializeField] private AssetGraphData currentGraph;
        [SerializeField] private SerializedObject serializedObject;
        [SerializeField] private AssetGraphViewWindow currentView;

        public AssetGraphData CurrentGraph => currentGraph;

        public static void Open(AssetGraphData asset)
        {
            foreach (var window in Resources.FindObjectsOfTypeAll<AssetGraphEditorWindow>())
            {
                if (window.CurrentGraph == asset)
                {
                    window.Focus();
                    return;
                }
            }
            
            var newWindow = CreateWindow<AssetGraphEditorWindow>(typeof(SceneView));
            newWindow.Initialize(asset);
        }

        private void Initialize(AssetGraphData asset)
        {
            titleContent = new GUIContent(
                asset.name,
                EditorGUIUtility.ObjectContent(null, typeof(AssetGraphData)).image
            );
            Load(asset);
            Focus();
        }

        private void OnEnable()
        {
            if (currentGraph != null)
            {
                DrawGraph();
            }
        }

        private void OnGUI()
        {
            if (currentGraph != null)
            {
                hasUnsavedChanges = EditorUtility.IsDirty(currentGraph);
            }
        }

        public void Load(AssetGraphData asset)
        {
            currentGraph = asset;
            DrawGraph();
        }

        private void DrawGraph()
        {
            serializedObject = new SerializedObject(currentGraph);
            currentView = new AssetGraphViewWindow(serializedObject, this)
            {
                graphViewChanged = OnGraphChanged
            };

            rootVisualElement.Clear();
            rootVisualElement.style.flexGrow = 1;
            rootVisualElement.Add(currentView);
        }

        private GraphViewChange OnGraphChanged(GraphViewChange graphViewChange)
        {
            EditorUtility.SetDirty(currentGraph);
            return graphViewChange;
        }
    }
}